import "reflect-metadata";
import express, { NextFunction, Express } from "express";
import { Response, Request } from "../global/types/express";
import { ParameterConfiguration, IController } from "../decorators";
import { Config } from "../configs/config";
import { IAccessApi, ICallback } from "../authorization";
interface IConfigApp {
    middlwareAuthExtend?: ICallback;
    logDocApi?: boolean;
}
declare class HttpException extends Error {
    statusCode: number;
    message: string;
    constructor(statusCode: number, message: string);
}
export declare const expressApp: typeof express;
export declare class Application {
    readonly config: Config;
    readonly isTest?: boolean | undefined;
    app: Express;
    private info;
    constructor(config: Config, isTest?: boolean | undefined);
    inital(): void;
    errorHandler: (error: HttpException, request: Request, response: Response, next: NextFunction) => void;
    generateRole(configRole: string): Promise<void>;
    generatePermisson(roleName: string, api: IAccessApi): Promise<void>;
    publishDocApi(): void;
    /**
     *
     * @param req Request of express
     * @param res Response of express
     * @param next Next function of express
     * @param paramConfigs Parameters
     * @param handlerName The name of the handler
     * @param controller The controller
     */
    apiHandler(req: Request, res: Response, next: NextFunction, paramConfigs: ParameterConfiguration[], handlerName: string, controller: IController<any>): void;
    onExit(): void;
    runRouter(callback: ICallback | null): void;
    runScheduler(): void;
    handleOnExist(): Promise<void>;
    createServer(port: number): Promise<void>;
    run(config: IConfigApp): Promise<void>;
}
export {};
