"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JwtService = exports.getAccessToken = exports.comparePassword = exports.expireTime = void 0;
const bcryptjs_1 = __importDefault(require("bcryptjs"));
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const helpers_1 = require("../helpers");
const errorBuilder_1 = require("../errorBuilder");
const role_model_1 = require("../authorization/RoleAndPermissions/role.model");
const injector_1 = require("../di/injector");
const configuration_1 = require("../configs/configuration");
exports.expireTime = 1;
const expiresIn = `${exports.expireTime}d`;
const comparePassword = (passwordAttempt, user) => {
    return new Promise((resolve, reject) => {
        bcryptjs_1.default.compare(passwordAttempt, user.password, (err, isMatch) => err ? reject(err) : resolve(isMatch));
    });
};
exports.comparePassword = comparePassword;
function getAccessToken(user) {
    return __awaiter(this, void 0, void 0, function* () {
        const role = yield (0, helpers_1.getQueryModel)(role_model_1.Role).findById((0, helpers_1.objectId)(user.roleId));
        if (!role) {
            throw errorBuilder_1.ErrorBuilder.notFound("Role does not exist");
        }
        const config = injector_1.injector.get(configuration_1.Configuration);
        const token = jsonwebtoken_1.default.sign({ id: user.id, roleId: role.id, roleName: role.name }, config.JWT_SECRET, { expiresIn: expiresIn });
        return token;
    });
}
exports.getAccessToken = getAccessToken;
var jwtService_1 = require("./jwtService");
Object.defineProperty(exports, "JwtService", { enumerable: true, get: function () { return jwtService_1.JwtService; } });
