"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JwtService = void 0;
const configuration_1 = require("../configs/configuration");
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const injector_1 = require("../di/injector");
const helpers_1 = require("../helpers");
const authorization_1 = require("../authorization");
const errorBuilder_1 = require("../errorBuilder");
const decorators_1 = require("../decorators");
let JwtService = exports.JwtService = class JwtService {
    constructor() {
        this.expiresAccessToken = "1d";
        this.expiresRefreshToken = "7d";
    }
    setExpiresAccessToken(expiresIn) {
        this.expiresAccessToken = expiresIn;
    }
    setexpiresRefreshToken(expiresIn) {
        this.expiresRefreshToken = expiresIn;
    }
    createToken(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const role = yield (0, helpers_1.getQueryModel)(authorization_1.Role).findById(user.roleId);
            if (!role) {
                throw errorBuilder_1.ErrorBuilder.notFound("Role does not exist");
            }
            const config = injector_1.injector.get(configuration_1.Configuration);
            const accessToken = jsonwebtoken_1.default.sign({ id: user.id || user._id, roleId: role.id, roleName: role.name }, config.JWT_SECRET, { expiresIn: this.expiresAccessToken });
            const refreshToken = jsonwebtoken_1.default.sign({ id: user.id || user._id, roleId: role.id, roleName: role.name }, config.JWT_SECRET, { expiresIn: this.expiresRefreshToken });
            return { accessToken, refreshToken };
        });
    }
    refreshToken(refreshToken) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = injector_1.injector.get(configuration_1.Configuration);
            const user = jsonwebtoken_1.default.verify(refreshToken, config.JWT_SECRET);
            const accessToken = jsonwebtoken_1.default.sign({ id: user.id, roleId: user.roleId, roleName: user.roleName }, config.JWT_SECRET, { expiresIn: this.expiresAccessToken });
            return accessToken;
        });
    }
    verify(token) {
        const config = injector_1.injector.get(configuration_1.Configuration);
        const user = jsonwebtoken_1.default.verify(token, config.JWT_SECRET);
        return user;
    }
};
exports.JwtService = JwtService = __decorate([
    (0, decorators_1.Register)()
], JwtService);
