"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UploadFile = exports.Upload = exports.ReqAuth = exports.Query = exports.Params = exports.Req = exports.Body = exports.extractParameters = void 0;
const constants_1 = require("../metadata/constants");
const interfaces_1 = require("./interfaces");
const createPipesRouteParamDecorator = (type, isFormData) => (target, key, index) => {
    const args = Reflect.getMetadata(constants_1.MetadataKeys.ROUTE_ARGS_METADATA, target.constructor, key) || [];
    args.push({ type, index, contentType: isFormData ? "multipart/form-data" : "application/json" });
    Reflect.defineMetadata(constants_1.MetadataKeys.ROUTE_ARGS_METADATA, args, target.constructor, key);
    // validate
    const indices = Reflect.getMetadata(`validate_${key}_parameters`, target, key) || [];
    indices.push(index);
    Reflect.defineMetadata(`validate_${key}_parameters`, indices, target, key);
};
function extractParameters(req, res, next, params) {
    if (!params || !params.length) {
        return [req];
    }
    const args = [];
    for (const { optionUpload, index, type } of params) {
        switch (type) {
            case interfaces_1.RouteParamtypes.REQUEST:
                args[index] = req;
                break;
            case interfaces_1.RouteParamtypes.PARAMS:
                args[index] = req.params;
                break;
            case interfaces_1.RouteParamtypes.QUERY:
                const query = Object.assign({}, req.query);
                Object.keys(query).forEach((key) => {
                    query[key] = JSON.parse(query[key]);
                });
                args[index] = query;
                break;
            case interfaces_1.RouteParamtypes.REQUEST_AUTH:
                args[index] = req.user;
                break;
            case interfaces_1.RouteParamtypes.BODY:
                args[index] = req.body;
                break;
            case interfaces_1.RouteParamtypes.UPLOAD:
                args[index] = req.file;
                break;
        }
    }
    return args;
}
exports.extractParameters = extractParameters;
const Body = (options = { isFormData: false }) => createPipesRouteParamDecorator(interfaces_1.RouteParamtypes.BODY, options === null || options === void 0 ? void 0 : options.isFormData);
exports.Body = Body;
const Req = () => createPipesRouteParamDecorator(interfaces_1.RouteParamtypes.REQUEST);
exports.Req = Req;
const Params = () => createPipesRouteParamDecorator(interfaces_1.RouteParamtypes.PARAMS);
exports.Params = Params;
const Query = () => createPipesRouteParamDecorator(interfaces_1.RouteParamtypes.QUERY);
exports.Query = Query;
const ReqAuth = () => createPipesRouteParamDecorator(interfaces_1.RouteParamtypes.REQUEST_AUTH);
exports.ReqAuth = ReqAuth;
const Upload = () => createPipesRouteParamDecorator(interfaces_1.RouteParamtypes.UPLOAD);
exports.Upload = Upload;
const UploadFile = (optionUpload) => {
    return (target, key, index) => {
        const args = Reflect.getMetadata(constants_1.MetadataKeys.ROUTE_ARGS_METADATA, target.constructor, key) || [];
        args.push({ type: interfaces_1.RouteParamtypes.UPLOAD, optionUpload, index, contentType: "multipart/form-data" });
        Reflect.defineMetadata(constants_1.MetadataKeys.ROUTE_ARGS_METADATA, args, target.constructor, key);
    };
};
exports.UploadFile = UploadFile;
