"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Del = exports.Patch = exports.Put = exports.Post = exports.Get = void 0;
const constants_1 = require("../metadata/constants");
const validateValueClass_1 = require("../helpers/validateValueClass");
const methodDecoratorFactory = (method) => {
    return (path, config = {
        isPublic: false,
        isAbsolute: false,
        allowFullParam: false
    }) => {
        return (target, propertyKey, descriptor) => {
            const controllerClass = target.constructor;
            const routers = Reflect.hasMetadata(constants_1.MetadataKeys.ROUTERS, controllerClass)
                ? Reflect.getMetadata(constants_1.MetadataKeys.ROUTERS, controllerClass)
                : [];
            const checkRouterExist = routers.find((router) => router.method == method && router.path == path);
            if (checkRouterExist) {
                throw new Error(`router path: ${path} exist`);
            }
            routers.push(Object.assign({ method,
                path, handlerName: propertyKey }, config));
            Reflect.defineMetadata(constants_1.MetadataKeys.ROUTERS, routers, controllerClass);
            // validate param
            const originalMethod = descriptor.value;
            const indices = Reflect.getMetadata(`validate_${propertyKey}_parameters`, target, propertyKey);
            const types = Reflect.getMetadata(constants_1.MetadataKeys.PARAM_TYPE, target, propertyKey);
            descriptor.value = function (...args) {
                if (Array.isArray(indices)) {
                    for (let i = 0; i < args.length; i++) {
                        if (indices.indexOf(i) !== -1) {
                            let value = args[i];
                            let validatorClass = types[i];
                            // if the validator class is array, we want to validate each item in the array
                            (0, validateValueClass_1.validationFieldErrors)(validatorClass, value, "", config.allowFullParam);
                        }
                    }
                }
                return originalMethod.apply(this, args);
            };
            return descriptor;
        };
    };
};
exports.Get = methodDecoratorFactory(constants_1.MethodRouter.GET);
exports.Post = methodDecoratorFactory(constants_1.MethodRouter.POST);
exports.Put = methodDecoratorFactory(constants_1.MethodRouter.PUT);
exports.Patch = methodDecoratorFactory(constants_1.MethodRouter.PATCH);
exports.Del = methodDecoratorFactory(constants_1.MethodRouter.DELETE);
