"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CronExpression = exports.Scheduler = void 0;
const di_1 = require("../di");
const metadata_1 = require("../global/metadata");
const constants_1 = require("../metadata/constants");
const node_cron_1 = require("node-cron");
const errorBuilder_1 = require("../errorBuilder");
function Scheduler() {
    return (constructorFn) => {
        (0, di_1.getServiceProvider)().register(constructorFn);
        (0, metadata_1.getMetadataScheduler)().push(constructorFn);
    };
}
exports.Scheduler = Scheduler;
function CronExpression(expression, option) {
    if (!(0, node_cron_1.validate)(expression)) {
        throw errorBuilder_1.ErrorBuilder.validator('Expression is invalid');
    }
    return function (target, name) {
        const controllerClass = target.constructor;
        const tasks = Reflect.hasMetadata(constants_1.MetadataKeys.SCHEDULER, controllerClass)
            ? Reflect.getMetadata(constants_1.MetadataKeys.SCHEDULER, controllerClass)
            : [];
        tasks.push({
            handlerName: name,
            expression,
            option
        });
        Reflect.defineMetadata(constants_1.MetadataKeys.SCHEDULER, tasks, controllerClass);
    };
}
exports.CronExpression = CronExpression;
