"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getServiceProvider = exports.ServiceProvider = exports.InjectionMode = void 0;
const errorBuilder_1 = require("../errorBuilder");
const injector_1 = require("./injector");
const constants_1 = require("../metadata/constants");
var InjectionMode;
(function (InjectionMode) {
    InjectionMode["TRANSIENT"] = "transient";
    InjectionMode["SINGLETON"] = "singleton";
})(InjectionMode || (exports.InjectionMode = InjectionMode = {}));
class ServiceProvider {
    constructor() {
        this.app = [];
        this.injectableRegistry = injector_1.injector;
    }
    register(constructorFn, mode = InjectionMode.SINGLETON, alias) {
        const dependencies = Reflect.getMetadata(constants_1.MetadataKeys.PARAM_TYPE, constructorFn) || [];
        const singletonInstance = mode == InjectionMode.SINGLETON ? this.injectableRegistry.resolve(constructorFn, false, alias) : undefined;
        this.app.push({
            constructorFn,
            mode,
            dependencies,
            singletonInstance,
            alias
        });
        return this;
    }
    getInjector() {
        return this.injectableRegistry;
    }
    getInfo(constructorFn) {
        const inject = this.app.find(e => e.constructorFn == constructorFn);
        if (!inject) {
            throw errorBuilder_1.ErrorBuilder.notFound("Inject not found");
        }
        return inject;
    }
}
exports.ServiceProvider = ServiceProvider;
const defaultContainer = new ServiceProvider();
function getServiceProvider() {
    return defaultContainer;
}
exports.getServiceProvider = getServiceProvider;
