"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validationColumnErrors = exports.validationFieldErrors = void 0;
const mongodb_1 = require("mongodb");
const errorBuilder_1 = require("../errorBuilder");
const metadata_1 = require("../global/metadata");
const constants_1 = require("../metadata/constants");
const regexPattern_1 = require("./regexPattern");
// validate Field in DTO
function validationFieldErrors(validatorClass, val, prefix = "", allowFullParam = false) {
    const fields = (0, metadata_1.getMetadataDTO)().getFieldDTO(validatorClass);
    if (!fields.length)
        return;
    const nameOfFields = (0, metadata_1.getMetadataDTO)().getNameFieldDTO(validatorClass);
    const checkKeyExist = Object.keys(val).filter((key) => {
        return !nameOfFields.includes(key);
    });
    if (checkKeyExist.length && !allowFullParam) {
        throw errorBuilder_1.ErrorBuilder.validator(`Key ${checkKeyExist} not allow`);
    }
    fields.forEach((field) => {
        var _a;
        if (field.isRequire && val[field.name] === undefined) {
            throw errorBuilder_1.ErrorBuilder.validator(`Key ${prefix}${field.name} is required`);
        }
        if (val[field.name]) {
            if (field.isEnum) {
                const valueEnums = Object.values((_a = field.vectorProps) === null || _a === void 0 ? void 0 : _a.elementDataType);
                if (field.type == Array) {
                    if (!Array.isArray(val[field.name])) {
                        throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} must be Array`);
                    }
                    val[field.name].forEach((e, index) => {
                        if (!valueEnums.includes(e)) {
                            throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name}[${index}] is not valid`);
                        }
                    });
                }
                else {
                    if (!valueEnums.includes(val[field.name])) {
                        throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} is not valid`);
                    }
                }
            }
            else {
                switch (field.type) {
                    case String:
                        if (typeof val[field.name] !== "string") {
                            throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} must be string`);
                        }
                        if (field.isEmail && !val[field.name].match(regexPattern_1.emailRegex)) {
                            throw errorBuilder_1.ErrorBuilder.validator(`Key ${prefix}${field.name} is not a valid email address`);
                        }
                        if (field.isPhone && !val[field.name].match(regexPattern_1.phoneRegex)) {
                            throw errorBuilder_1.ErrorBuilder.validator(`Key ${prefix}${field.name} is not a valid phone`);
                        }
                        if (field.regex && !val[field.name].match(field.regex)) {
                            throw errorBuilder_1.ErrorBuilder.validator(`Key ${prefix}${field.name} is not a valid regex`);
                        }
                        if (field.isRequire && (val[field.name] === undefined || val[field.name] === '')) {
                            throw errorBuilder_1.ErrorBuilder.validator(`Key ${prefix}${field.name} is required`);
                        }
                        break;
                    case Number:
                        if (typeof val[field.name] !== "number") {
                            throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} must be number`);
                        }
                        break;
                    case Boolean:
                        if (typeof val[field.name] !== "boolean") {
                            throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} must be boolean`);
                        }
                        break;
                    case mongodb_1.ObjectId:
                        if (typeof val[field.name] !== "string") {
                            throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} must be ObjectId string`);
                        }
                        if (field.isRequire && (val[field.name] === undefined || val[field.name] === '')) {
                            throw errorBuilder_1.ErrorBuilder.validator(`Key ${prefix}${field.name} is required`);
                        }
                        break;
                    case Array:
                        validateElementInArray(val, field, prefix);
                        break;
                    default:
                        // field.type is class
                        validationFieldErrors(field.type, val[field.name], `${prefix}${field.name}.`);
                        break;
                }
            }
        }
    });
}
exports.validationFieldErrors = validationFieldErrors;
function validateElementInArray(val, field, prefix = "") {
    var _a, _b;
    if (!Array.isArray(val[field.name])) {
        throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} must be Array`);
    }
    if ((_a = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _a === void 0 ? void 0 : _a.elementDataType) {
        switch ((_b = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _b === void 0 ? void 0 : _b.elementDataType) {
            case constants_1.DataType.BOOLEAN:
                val[field.name].forEach((e, index) => {
                    if (typeof e !== "boolean") {
                        throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name}[${index}] must be boolean`);
                    }
                });
                break;
            case constants_1.DataType.NUMBER:
                val[field.name].forEach((e, index) => {
                    if (typeof e !== "number") {
                        throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name}[${index}] must be number`);
                    }
                });
                break;
            case mongodb_1.ObjectId:
            case constants_1.DataType.STRING:
                val[field.name].forEach((e, index) => {
                    if (typeof e !== "string") {
                        throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name}[${index}] must be string`);
                    }
                });
                break;
            default:
                val[field.name].forEach((e, index) => {
                    var _a;
                    validationFieldErrors((_a = field.vectorProps) === null || _a === void 0 ? void 0 : _a.elementDataType, e, `${prefix}${field.name}[${index}].`);
                });
                break;
        }
    }
}
// validate value of table insert to DB
function validationColumnErrors(validatorClass, val, prefix = "", isUpdate = false) {
    const fields = (0, metadata_1.getMetadataTable)().getColumns(validatorClass);
    const nameOfFields = fields.map(e => e.name);
    const checkKeyExist = Object.keys(val).filter((key) => {
        return !nameOfFields.includes(key);
    });
    if (checkKeyExist.length) {
        throw errorBuilder_1.ErrorBuilder.validator(`Key ${checkKeyExist} not allow`);
    }
    fields.forEach((field) => {
        var _a;
        if (field.isRequire && field.default === undefined && val[field.name] === undefined && !isUpdate && !field.isPrimaryKey) {
            throw errorBuilder_1.ErrorBuilder.validator(`Key ${prefix}${field.name} is required`);
        }
        if (val[field.name]) {
            if (field.isEnum) {
                const valueEnums = Object.values((_a = field.vectorProps) === null || _a === void 0 ? void 0 : _a.elementDataType);
                if (field.type == Array) {
                    if (!Array.isArray(val[field.name])) {
                        throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} must be Array`);
                    }
                    val[field.name].forEach((e, index) => {
                        if (!valueEnums.includes(e)) {
                            throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name}[${index}] is not valid`);
                        }
                    });
                }
                else {
                    if (!valueEnums.includes(val[field.name])) {
                        throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} is not valid`);
                    }
                }
            }
            else {
                switch (field.type) {
                    case String:
                        if (typeof val[field.name] !== "string") {
                            throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} must be string`);
                        }
                        if (field.isRequire && field.default === undefined && (val[field.name] === undefined || val[field.name] === '' && !isUpdate)) {
                            throw errorBuilder_1.ErrorBuilder.validator(`Key ${prefix}${field.name} is required`);
                        }
                        break;
                    case Number:
                        if (typeof val[field.name] !== "number") {
                            throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} must be number`);
                        }
                        break;
                    case Boolean:
                        if (typeof val[field.name] !== "boolean") {
                            throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} must be boolean`);
                        }
                        break;
                    case mongodb_1.ObjectId:
                        if ((typeof val[field.name] !== "string" && typeof val[field.name] != "object") && !field.isPrimaryKey) {
                            throw errorBuilder_1.ErrorBuilder.validator(`Type ${prefix}${field.name} must be ObjectId string`);
                        }
                        break;
                    case Array:
                        validateElementInArray(val, field, prefix);
                        break;
                    default:
                        // field.type is class
                        validationColumnErrors(field.type, val[field.name], `${prefix}${field.name}.`, isUpdate);
                        break;
                }
            }
        }
    });
}
exports.validationColumnErrors = validationColumnErrors;
