"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SqlMigrator = void 0;
const AbstractSqlMigrator_1 = require("./AbstractSqlMigrator");
const utils_1 = require("./utils");
const fs = __importStar(require("fs"));
const child_process_1 = require("child_process");
const path = __importStar(require("path"));
const errorBuilder_1 = require("../errorBuilder");
const commons_1 = require("../commons");
class SqlMigrator extends AbstractSqlMigrator_1.AbstractSqlMigrator {
    constructor(connection, dir, metaTable = "SequelizeMeta") {
        super();
        this.connection = connection;
        this.dir = dir;
        this.metaTable = metaTable;
        if (!["mysql", "postgres"].includes(connection.type)) {
            throw errorBuilder_1.ErrorBuilder.validator("Type database is not supported.");
        }
        this.sequelize = this.connection.db;
        this.migrationModel = this.sequelize.define(this.metaTable, utils_1.metaTableDefinition, {
            tableName: this.metaTable,
            freezeTableName: true,
            name: {
                singular: this.metaTable,
                plural: this.metaTable,
            },
            timestamps: false,
        });
    }
    initMigrationTables(queryInterface) {
        return __awaiter(this, void 0, void 0, function* () {
            yield queryInterface.createTable(this.metaTable, utils_1.metaTableDefinition);
        });
    }
    getAllMigrations(queryInterface) {
        return __awaiter(this, void 0, void 0, function* () {
            let tableExist = true;
            try {
                yield queryInterface.describeTable(this.metaTable);
            }
            catch (err) {
                tableExist = false;
            }
            if (!tableExist) {
                //-- init the schema
                yield this.initMigrationTables(queryInterface);
            }
            const migrations = (yield this.migrationModel.findAll({ attributes: ["name"], raw: true, order: [["name", "ASC"]] }));
            return migrations.map((mi) => mi.name);
        });
    }
    migrate(modules) {
        return __awaiter(this, void 0, void 0, function* () {
            const migrations = Object.keys(modules);
            const queryInterface = this.sequelize.getQueryInterface();
            const migrated = yield this.getAllMigrations(queryInterface);
            const notMigrated = migrations.filter((fileName) => !migrated.some((m) => fileName.startsWith(m)));
            for (const migrationName of notMigrated) {
                const tx = yield this.sequelize.transaction();
                try {
                    //-- run migration
                    const module = modules[migrationName];
                    yield module.up(queryInterface, tx);
                    yield this.migrationModel.bulkCreate([{ name: migrationName }], { transaction: tx });
                    console.log((0, commons_1.csuccess)(`up ${migrationName} successfully`));
                    yield tx.commit();
                }
                catch (err) {
                    console.log((0, commons_1.cerror)(`up ${migrationName} failed with error:`));
                    console.log(err);
                    yield tx.rollback();
                    break;
                }
            }
        });
    }
    rollback(modules, step = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            const migrations = Object.keys(modules);
            const queryInterface = this.sequelize.getQueryInterface();
            let migrated = yield this.getAllMigrations(queryInterface);
            migrated = migrated.reverse().filter((value, index) => index < step);
            const migrateRollback = migrated.filter((fileName) => migrations.some((m) => fileName.startsWith(m)));
            for (const migrationName of migrateRollback) {
                const tx = yield this.sequelize.transaction();
                try {
                    //-- run migration
                    const module = modules[migrationName];
                    yield module.down(queryInterface, tx);
                    yield this.migrationModel.destroy({
                        where: {
                            name: migrationName
                        },
                        transaction: tx
                    });
                    yield tx.commit();
                    console.log((0, commons_1.csuccess)(`down ${migrationName} successfully`));
                }
                catch (err) {
                    console.log((0, commons_1.cerror)(`down ${migrationName} failed with error:`));
                    console.log(err);
                    yield tx.rollback();
                    break;
                }
            }
        });
    }
    generateDBMetadata(outputFile) {
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise((resolve, reject) => {
                fs.writeFile(outputFile, utils_1.MIGRATION_TEMPLATE, "utf8", (err) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    // file written successfully
                    (0, child_process_1.exec)(`npx prettier --write ${outputFile}`, (err) => {
                        if (err) {
                            reject(err);
                            return;
                        }
                        console.log("success");
                        resolve(1);
                    });
                });
            });
        });
    }
    execuseCommand() {
        return __awaiter(this, void 0, void 0, function* () {
            const command = process.env.npm_config_command;
            const dir = path.join(process.cwd(), this.dir);
            if (!fs.existsSync(dir)) {
                fs.mkdirSync(dir);
            }
            const files = fs.readdirSync(dir).sort();
            let modules = {};
            for (const file of files) {
                const migration = require(path.join(dir, file)).default;
                modules[file] = migration;
            }
            switch (command) {
                case utils_1.MigratorCommand.GENERATE:
                    let name = process.env.npm_config_name || "";
                    name = name.replace(/ /g, '_');
                    const now = Date.now();
                    const outputFile = path.join(dir, `${now}_${name}.ts`);
                    yield this.generateDBMetadata(outputFile);
                    console.log((0, commons_1.csuccess)(`create file ${now}_${name}.ts successfully`));
                    break;
                case undefined:
                case utils_1.MigratorCommand.UPGRADE:
                    yield this.migrate(modules);
                    break;
                case utils_1.MigratorCommand.DOWNGRADE:
                    const step = process.env.npm_config_step || 1;
                    yield this.rollback(modules, +step);
                    break;
                default:
                    console.log((0, commons_1.cerror)(errorBuilder_1.ErrorBuilder.notFound("Command not found")));
            }
        });
    }
}
exports.SqlMigrator = SqlMigrator;
