"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MigratorAplication = void 0;
require("reflect-metadata");
const dotenv_1 = __importDefault(require("dotenv"));
const injector_1 = require("../di/injector");
const configuration_1 = require("../configs/configuration");
const errorBuilder_1 = require("../errorBuilder");
const mongodb_1 = require("../connections/mongodb");
const sql_1 = require("../connections/sql");
const SqlMigrator_1 = require("./SqlMigrator");
dotenv_1.default.config();
class MigratorAplication {
    constructor(config, pathDir = "/migrations") {
        this.config = config;
        this.pathDir = pathDir;
        injector_1.injector.set(configuration_1.Configuration, config);
    }
    run(config) {
        return __awaiter(this, void 0, void 0, function* () {
            let db;
            switch (this.config.DB_CONFIG.TYPE) {
                case "mongodb":
                    db = yield (0, mongodb_1.connectMongoDb)({
                        database: this.config.DB_CONFIG.DATABASE,
                        host: this.config.DB_CONFIG.HOST,
                        port: this.config.DB_CONFIG.PORT,
                        username: this.config.DB_CONFIG.USERNAME,
                        password: this.config.DB_CONFIG.PASSWORD
                    });
                    break;
                case "mysql":
                    db = yield (0, sql_1.connectDatabaseSql)({
                        database: this.config.DB_CONFIG.DATABASE,
                        host: this.config.DB_CONFIG.HOST,
                        port: this.config.DB_CONFIG.PORT,
                        username: this.config.DB_CONFIG.USERNAME,
                        password: this.config.DB_CONFIG.PASSWORD,
                        type: "mysql",
                        isLog: config.isLog
                    });
                    break;
                case "postgres":
                    db = yield (0, sql_1.connectDatabaseSql)({
                        database: this.config.DB_CONFIG.DATABASE,
                        host: this.config.DB_CONFIG.HOST,
                        port: this.config.DB_CONFIG.PORT,
                        username: this.config.DB_CONFIG.USERNAME,
                        password: this.config.DB_CONFIG.PASSWORD,
                        type: "postgres",
                        isLog: config.isLog
                    });
                    break;
                default:
                    throw errorBuilder_1.ErrorBuilder.validator("No DB connection");
            }
            const dbConnection = {
                db: db,
                type: this.config.DB_CONFIG.TYPE
            };
            const sqlMigrator = new SqlMigrator_1.SqlMigrator(dbConnection, this.pathDir);
            yield sqlMigrator.execuseCommand();
        });
    }
}
exports.MigratorAplication = MigratorAplication;
