export type Op = "$eq" | "$ne" | "$lt" | "$lte" | "$gt" | "$gte";
export type OpSpecial = "$like" | "$notLike" | "$regex" | "$exits" | "$options";
export type OpArray = "$in" | "$nin";
export type MultipleOp = "$and" | "$or";
export type SingleOperator<T> = {
    [key in keyof T]?: {
        [op in Op]?: T[key];
    };
} & Record<string, {
    [op in Op]?: any;
}>;
export type SingleArrayOperator<T> = {
    [key in keyof T]?: {
        [op in OpArray]?: T[key][];
    };
} & Record<string, {
    [op in OpArray]?: any;
}>;
export type SpecialOperfator<T> = {
    [key in keyof T]?: {
        $like?: T[key] extends string ? string : never;
        $notLike?: T[key] extends string ? string : never;
        $regex?: T[key] extends string ? RegExp | string : never;
        $exists?: boolean;
        $options?: "i";
        $eq?: T[key];
        $lt?: T[key];
        $lte?: T[key];
        $gt?: T[key];
        $gte?: T[key];
    };
} & Record<string, {
    [op in OpSpecial]?: any;
}>;
export type MultipleOperator<T> = {
    [key in MultipleOp]?: (SingleOperator<T> | SingleArrayOperator<T> | SpecialOperfator<T> | MultipleOperator<T>)[];
};
export type SingleQueryCondition<T> = SingleOperator<T> | SingleArrayOperator<T> | SpecialOperfator<T>;
export type QueryCondition<T> = SingleQueryCondition<T> | MultipleOperator<T>;
export type Sort<T> = {
    [key in keyof T]?: SortDirection;
};
export type SortDirection = 1 | -1 | 'asc' | 'desc';
export type FindOptions<T> = {
    /** Sets the limit of documents returned in the query. */
    limit?: number;
    /** Set to sort the documents coming back from the query. Array of indexes, `[['a', 1]]` etc. */
    sort?: Sort<T>;
    /** Set to skip N documents ahead in your query (useful for pagination). */
    skip?: number;
    fromCRUD?: boolean;
};
export type FunctionQueryBuilder = "find" | "connection" | "db" | "collection" | "getTableName" | "getColumns" | "getNameColumns" | "getRelations" | "getPrimaryKey" | "getDB" | "with" | "select" | "findById" | "findOne" | "find" | "paginate" | "count" | "createOne" | "updateMany" | "delete" | "deleteMany" | "validateParamBeforeSaveDB" | "mappingDataRelation" | "prepareDataRelation";
export type KeyColumn<T> = Exclude<keyof T, FunctionQueryBuilder>;
export type KeyColumns<T> = KeyColumn<T>[];
export type ModelQuery<T> = {
    [key in KeyColumn<T>]: T[key];
};
