"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoQueryBuilder = void 0;
const mongodb_1 = require("mongodb");
const decorators_1 = require("../decorators");
const helpers_1 = require("../helpers");
const errorBuilder_1 = require("../errorBuilder");
const global_1 = require("../global");
const multiOps = ["$and", "$or"];
class MongoQueryBuilder {
    constructor(db, table) {
        this.db = db;
        this.table = table;
        this.withRelations = [];
        this.selectColumns = [];
    }
    getPrimaryKey() {
        const primaryKey = this.table.columns.find(e => e.isPrimaryKey);
        if (!primaryKey) {
            throw Error(`primary key not found`);
        }
        return primaryKey;
    }
    getCollection() {
        return this.db.collection(this.table.name);
    }
    generateWith(params, model) {
        const withRelations = [];
        const relations = (0, global_1.getMetadataTable)().getTable(model).relations;
        if (Array.isArray(params)) {
            params.forEach(e => {
                const findRelation = relations.find((relation) => relation.fieldName == e.model);
                if (findRelation) {
                    if (e.with) {
                        const modelChild = ((findRelation.type == decorators_1.TypeRelation.HAS_MANY || findRelation.type == decorators_1.TypeRelation.HAS_ONE)
                            ? findRelation.refs.modelChild()
                            : findRelation.refs.modelParent());
                        const withChild = this.generateWith(e.with, modelChild);
                        e.with = withChild;
                    }
                    withRelations.push(Object.assign(Object.assign({}, e), { relation: findRelation }));
                }
                else {
                    throw errorBuilder_1.ErrorBuilder.validator(`with ${e.model.toString()} invalid`);
                }
            });
        }
        else {
            const findRelation = relations.find((relation) => relation.fieldName == params.model);
            if (findRelation) {
                if (params.with) {
                    const modelChild = ((findRelation.type == decorators_1.TypeRelation.HAS_MANY || findRelation.type == decorators_1.TypeRelation.HAS_ONE)
                        ? findRelation.refs.modelChild()
                        : findRelation.refs.modelParent());
                    const withChild = this.generateWith(params.with, modelChild);
                    params.with = withChild;
                }
                withRelations.push(Object.assign(Object.assign({}, params), { relation: findRelation }));
            }
            else {
                throw errorBuilder_1.ErrorBuilder.validator(`with ${params.model.toString()} invalid`);
            }
        }
        return withRelations;
    }
    with(params, model) {
        this.withRelations = this.generateWith(params, model.constructor);
    }
    generateAggregate(withRelations, isFromCRUD) {
        const aggregate = [];
        withRelations.forEach(_with => {
            var _a, _b, _c, _d;
            const relation = _with.relation;
            let lookup;
            const fieldAllow = (0, global_1.getMetadataTable)()
                .getTable(relation.type == decorators_1.TypeRelation.BELONG_TO ? relation.refs.modelParent() : relation.refs.modelChild())
                .columns
                .filter(column => !column.isPassword || !isFromCRUD)
                .map(column => column.name.toString());
            _with.select = _with.select ? _with.select.filter(e => fieldAllow.includes(e.toString())) : fieldAllow;
            switch (relation.type) {
                case decorators_1.TypeRelation.BELONG_TO:
                    const tableParent = (0, global_1.getMetadataTable)().getTable(relation.refs.modelParent());
                    lookup = {
                        from: (0, helpers_1.convertToPlural)(relation.refs.modelParent().name),
                        localField: relation.refs.FK.toString(),
                        foreignField: (_a = tableParent.columns.find(e => e.isPrimaryKey)) === null || _a === void 0 ? void 0 : _a.name,
                        as: relation.fieldName,
                        pipeline: []
                    };
                    lookup.pipeline.push({
                        $project: (_b = _with.select) === null || _b === void 0 ? void 0 : _b.reduce((target, column) => (Object.assign(Object.assign({}, target), { [column]: 1 })), {})
                    });
                    if (_with.filter) {
                        lookup.pipeline.push({
                            $match: this.getQueryObjectFromQueryConditions(_with.filter)
                        });
                    }
                    if (_with.with) {
                        lookup.pipeline = [...lookup.pipeline, ...this.generateAggregate(Array.isArray(_with.with) ? _with.with : [_with.with], isFromCRUD)];
                    }
                    aggregate.push({
                        $lookup: lookup,
                    });
                    aggregate.push({
                        $unwind: {
                            path: `$${relation.fieldName}`,
                            preserveNullAndEmptyArrays: true,
                        },
                    });
                    aggregate.push({
                        $addFields: {
                            [`${relation.fieldName}`]: {
                                $ifNull: [`$${relation.fieldName}`, null]
                            }
                        },
                    });
                    break;
                case decorators_1.TypeRelation.HAS_ONE:
                    lookup = {
                        from: (0, helpers_1.convertToPlural)(relation.refs.modelChild().name),
                        localField: this.getPrimaryKey().name,
                        foreignField: relation.refs.FK.toString(),
                        as: relation.fieldName,
                        pipeline: [
                            {
                                $limit: 1
                            }
                        ]
                    };
                    lookup.pipeline.push({
                        $project: (_c = _with.select) === null || _c === void 0 ? void 0 : _c.reduce((target, column) => (Object.assign(Object.assign({}, target), { [column]: 1 })), {})
                    });
                    if (_with.filter) {
                        lookup.pipeline.push({
                            $match: this.getQueryObjectFromQueryConditions(_with.filter)
                        });
                    }
                    if (_with.with) {
                        lookup.pipeline = [...lookup.pipeline, ...this.generateAggregate(Array.isArray(_with.with) ? _with.with : [_with.with], isFromCRUD)];
                    }
                    aggregate.push({
                        $lookup: lookup,
                    });
                    aggregate.push({
                        $unwind: {
                            path: `$${relation.fieldName}`,
                            preserveNullAndEmptyArrays: true,
                        },
                    });
                    aggregate.push({
                        $addFields: {
                            [`${relation.fieldName}`]: {
                                $ifNull: [`$${relation.fieldName}`, null]
                            }
                        },
                    });
                    break;
                case decorators_1.TypeRelation.HAS_MANY:
                    lookup = {
                        from: (0, helpers_1.convertToPlural)(relation.refs.modelChild().name),
                        localField: this.getPrimaryKey().name,
                        foreignField: relation.refs.FK.toString(),
                        as: relation.fieldName,
                        pipeline: []
                    };
                    lookup.pipeline.push({
                        $project: (_d = _with.select) === null || _d === void 0 ? void 0 : _d.reduce((target, column) => (Object.assign(Object.assign({}, target), { [column]: 1 })), {})
                    });
                    if (_with.filter) {
                        lookup.pipeline.push({
                            $match: this.getQueryObjectFromQueryConditions(_with.filter)
                        });
                    }
                    if (_with.with) {
                        lookup.pipeline = [...lookup.pipeline, ...this.generateAggregate(Array.isArray(_with.with) ? _with.with : [_with.with], isFromCRUD)];
                    }
                    aggregate.push({
                        $lookup: lookup,
                    });
                    aggregate.push({
                        $addFields: {
                            [`${relation.fieldName}`]: `$${relation.fieldName}`
                        },
                    });
                    break;
                default:
                    break;
            }
        });
        return aggregate;
    }
    findByPk(pk) {
        return __awaiter(this, void 0, void 0, function* () {
            const fieldPk = this.table
                .columns
                .find(column => column.isPrimaryKey);
            if (!fieldPk) {
                throw errorBuilder_1.ErrorBuilder.validator(`primary key not found`);
            }
            switch (fieldPk.type) {
                case mongodb_1.ObjectId:
                    if (typeof pk === "string") {
                        pk = (0, helpers_1.objectId)(pk);
                    }
                    if (typeof pk === "number") {
                        throw errorBuilder_1.ErrorBuilder.validator(`type ${pk} is not a number`);
                    }
                    break;
                case String:
                    if (typeof pk !== "string") {
                        pk = pk.toString();
                    }
                    break;
                case Number:
                    if (typeof pk === "object") {
                        throw errorBuilder_1.ErrorBuilder.validator(`type ${pk} is not a object`);
                    }
                    pk = +pk;
                    break;
            }
            const data = yield this.find({
                [fieldPk.name]: { $eq: pk }
            });
            return data.length ? data[0] : null;
        });
    }
    find(filter, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const fieldAllow = this.table
                .columns
                .filter(column => !column.isPassword || !(options === null || options === void 0 ? void 0 : options.fromCRUD))
                .map(column => column.name.toString());
            const selectColumns = this.selectColumns.length ? this.selectColumns.filter(e => fieldAllow.includes(e.toString())).map(e => e.toString()) : fieldAllow;
            const aggregate = [
                {
                    $match: this.getQueryObjectFromQueryConditions(filter)
                },
                {
                    $project: selectColumns.reduce((target, column) => (Object.assign(Object.assign({}, target), { [column]: 1 })), {})
                },
                ...this.generateAggregate(this.withRelations, (options === null || options === void 0 ? void 0 : options.fromCRUD) || false),
            ];
            if (options === null || options === void 0 ? void 0 : options.sort) {
                aggregate.push({
                    $sort: Object.keys(options.sort).reduce((target, item) => {
                        if (typeof options.sort[item] == "string") {
                            target[item] = options.sort[item] == "asc" ? 1 : -1;
                        }
                        return target;
                    }, {})
                });
            }
            if (options === null || options === void 0 ? void 0 : options.skip) {
                aggregate.push({
                    $skip: options === null || options === void 0 ? void 0 : options.skip
                });
            }
            if (options === null || options === void 0 ? void 0 : options.limit) {
                aggregate.push({
                    $limit: options === null || options === void 0 ? void 0 : options.limit
                });
            }
            const query = this.getCollection().aggregate(aggregate);
            return query.toArray();
        });
    }
    paginate(filter, paginate, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const [data, totalCount] = yield Promise.all([
                this.find(filter, Object.assign(Object.assign({}, options), { limit: paginate.limit, skip: (paginate.page - 1) * paginate.limit })),
                this.count(filter),
            ]);
            return { data, totalCount };
        });
    }
    count(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getCollection().count(this.getQueryObjectFromQueryConditions(filter));
        });
    }
    createOne(item) {
        return __awaiter(this, void 0, void 0, function* () {
            const newDate = new Date().getTime();
            const dataInsert = yield this.getCollection().insertOne(Object.assign(Object.assign({}, item), { createdAt: newDate, updatedAt: newDate }));
            const data = yield this.find({ _id: { $eq: (0, helpers_1.objectId)(dataInsert.insertedId) } }, { fromCRUD: true });
            return data[0];
        });
    }
    updateMany(filter, item) {
        return __awaiter(this, void 0, void 0, function* () {
            const newDate = new Date().getTime();
            return this.getCollection().updateMany(this.getQueryObjectFromQueryConditions(filter), {
                $set: Object.assign(Object.assign({}, item), { updatedAt: newDate }),
            });
        });
    }
    deleteOne(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getCollection().deleteOne(this.getQueryObjectFromQueryConditions(filter));
        });
    }
    deleteMany(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getCollection().deleteMany(this.getQueryObjectFromQueryConditions(filter));
        });
    }
    select(selectColumns) {
        this.selectColumns = selectColumns;
    }
    reset() {
        this.withRelations = [];
        this.selectColumns = [];
    }
    mapToMongoOperator(op, attribute, value) {
        switch (op) {
            case "$like":
                return {
                    [attribute]: {
                        $regex: value,
                        $options: "i"
                    }
                };
            default:
                return {
                    [attribute]: {
                        [op]: value
                    }
                };
        }
    }
    getQueryObjectFromQueryConditions(queries) {
        if (!queries) {
            return {};
        }
        const keys = Object.keys(queries);
        if (!keys.length) {
            return {};
        }
        const ops = keys.map((key) => {
            if (multiOps.includes(key)) {
                return {
                    [key]: queries[key].map((o) => this.getQueryObjectFromQueryConditions(o))
                };
            }
            else {
                const operator = Object.keys(queries[key])[0];
                const field = key;
                const value = queries[field][operator];
                return this.mapToMongoOperator(operator, field, value);
            }
        });
        return ops.length > 1 ? { $and: ops } : ops[0];
    }
}
exports.MongoQueryBuilder = MongoQueryBuilder;
