"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApplicationWebSocket = void 0;
require("reflect-metadata");
const uuidv4_1 = require("uuidv4");
const ServerSocket_1 = require("./ServerSocket");
class ApplicationWebSocket {
    constructor(connection, wss, redis) {
        this.connection = connection;
        this.wss = wss;
        this.redis = redis;
    }
    run() {
        const self = this;
        this.wss.on("connection", function connection(client, request) {
            client.id = (0, uuidv4_1.uuid)();
            try {
                const socket = new ServerSocket_1.ServerSocket(client, request);
                self.connection.onSocketConnect(socket);
                client.on('error', console.error);
                client.on('message', function message(message) {
                    const data = JSON.parse(message);
                    self.connection.onMessage(socket, data);
                });
                client.on('close', function close(code, reason) {
                    return __awaiter(this, void 0, void 0, function* () {
                        self.connection.onSocketDisconnect(socket);
                        const channels = yield self.redis.client.hGetAll("_SOCKET");
                        const promises = [];
                        Object.keys(channels).forEach((key) => {
                            const valueChanels = channels[key];
                            let socketChannel;
                            if (valueChanels) {
                                socketChannel = JSON.parse(valueChanels);
                            }
                            if (socketChannel) {
                                socketChannel.value = socketChannel.value.filter(socketId => socketId != socket.getId());
                                socketChannel.updatedAt = new Date().toISOString();
                                promises.push(self.redis.client.hSet("_SOCKET", key, JSON.stringify(socketChannel)));
                            }
                        });
                        yield Promise.all(promises);
                    });
                });
            }
            catch (error) {
                client.send(error.message);
            }
        });
    }
}
exports.ApplicationWebSocket = ApplicationWebSocket;
