"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SocketManager = void 0;
class SocketManager {
    constructor(redis, websocket) {
        this.redis = redis;
        this.websocket = websocket;
        this.modelRedis = "_SOCKET";
    }
    static getInstance(redis, websocket) {
        if (!this.instance) {
            this.instance = new SocketManager(redis, websocket);
        }
        return this.instance;
    }
    addSocketToChannel(socketId, channels) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(channels.map((channel) => __awaiter(this, void 0, void 0, function* () {
                const valueChanels = yield this.redis.client.hGet(this.modelRedis, channel);
                let socketChannel;
                if (valueChanels) {
                    socketChannel = JSON.parse(valueChanels);
                }
                else {
                    socketChannel = {
                        updatedAt: new Date().toISOString(),
                        value: []
                    };
                }
                if (socketChannel) {
                    if (!socketChannel.value.includes(socketId)) {
                        socketChannel.value.push(socketId);
                    }
                    socketChannel.updatedAt = new Date().toISOString();
                    yield this.redis.client.hSet(this.modelRedis, channel, JSON.stringify(socketChannel));
                }
                return true;
            })));
        });
    }
    removeSocketFromChannel(socketId, channels) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(channels.map((channel) => __awaiter(this, void 0, void 0, function* () {
                const valueChanels = yield this.redis.client.hGet(this.modelRedis, channel);
                let socketChannel;
                if (valueChanels) {
                    socketChannel = JSON.parse(valueChanels);
                }
                if (socketChannel) {
                    socketChannel.value = socketChannel.value.filter(e => socketId != e);
                    socketChannel.updatedAt = new Date().toISOString();
                    yield this.redis.client.hSet(this.modelRedis, channel, JSON.stringify(socketChannel));
                }
                yield this.redis.client.hSet(this.modelRedis, channel, JSON.stringify(socketChannel));
                return 1;
            })));
        });
    }
    broadcastToChannel(channel, data) {
        return __awaiter(this, void 0, void 0, function* () {
            const dataRedis = yield this.redis.client.hGet(this.modelRedis, channel);
            if (dataRedis) {
                const channels = JSON.parse(dataRedis);
                if (Array.isArray(channels === null || channels === void 0 ? void 0 : channels.value)) {
                    this.websocket.clients.forEach((client) => {
                        if (channels === null || channels === void 0 ? void 0 : channels.value.includes(client.id)) {
                            client.send(data);
                        }
                    });
                }
            }
        });
    }
}
exports.SocketManager = SocketManager;
